package scheme.interp;

import java.util.HashMap;

import scheme.util.*;

// replaced SchemeValue environment with a generic
// Storage based environment, similar to a Map, but
// without all the java Map features.

public class Env {
    private HashMap hash;
    
    public Env() {
        this.hash = new HashMap();
    }
    
    protected Env (HashMap hash) {
        this.hash = hash;
    }

    public Object find(String x) throws VisitE {
        // if our map has this key, return the value
        if (this.hash.containsKey(x)) {
            return this.hash.get(x);
        }
        
        // otherwise, it's unbound. throw an exception
        throw new VisitE("Unbound variable " + x);
    }

    public void set(String x, Object v) throws VisitE {
        if (!this.hash.containsKey(x)) {
            throw new VisitE("Unbound variable " + x);
        }
        
        // rebind the key
        this.hash.put(x,v);
    }

    public Env add (String x, Object v) {
        HashMap newenv = new HashMap(this.hash);
        
        // add the binding to the new environment
        newenv.put(x,v);
        
        // return a new environment
        return new Env(newenv);
    }
    
    public void setAll (List xs, List vs) throws VisitE {
        while (! (xs instanceof EmptyL)) {
            String x = (String) xs.hd();
            //SchemeValue v = (SchemeValue) vs.hd();
            this.set(x,vs.hd());
            xs = xs.tl();
            vs = vs.tl();
        }
    }

    public Env addAll (List xs, List vs) {
        Env result = this;
        while (! (xs instanceof EmptyL)) {
            String x = (String) xs.hd();
            //SchemeValue v = (SchemeValue) vs.hd();
            result = result.add(x,vs.hd());
            xs = xs.tl();
            vs = vs.tl();
        }
        return result;
    }

    public Env addAllVoid (List xs) {
        Env result = this;
        SchemeValue v = new VoidV();
        while (! (xs instanceof EmptyL)) {
            String x = (String) xs.hd();
            result = result.add(x,v);
            xs = xs.tl();
        }
        return result;
    }
}
